@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -32,6 +32,7 @@
 #include "base/RndPermut.h"
 #include "base/UniqId.h"
 #include "base/BStream.h"
+#include "base/IpRange.h"
 #include "base/polyLogTags.h"
 #include "base/polyLogCats.h"
 #include "runtime/AddrMap.h"
@@ -338,7 +339,6 @@ void PolyApp::configureHosts() {
 	// assign protocols and SSL wraps to servers, including proxies
 	int sslServerCount = 0;
 	Array<int> forAddrMap(srvCount);
-	SrvSharedCfgs srvCfgs;
 	for (AgentAddrIter i(agents, ServerSym::TheType); i; ++i) {
 		int idx = -1;
 		const NetAddr &host = i.address();
@@ -364,7 +364,7 @@ void PolyApp::configureHosts() {
 
 		const ServerSym *srv =
 			&static_cast<const ServerSym&>(i.agent()->cast(ServerSym::TheType));
-		SrvCfg *srvCfg = srvCfgs.getConfig(srv);
+		SrvCfg *const srvCfg = TheSrvSharedCfgs.getConfig(srv);
 		hostCfg->theProtocol =
 			static_cast< Agent::Protocol >(srvCfg->selectProtocol());
 		if (srvCfg->sslActive(hostCfg->theProtocol)) {
@@ -473,7 +473,6 @@ void PolyApp::makeAddresses() {
 	while (hosts.count()) delete hosts.pop();
 	while (agents.count()) delete agents.pop();
 	theIfaces.reset();
-	while (theCleanIfaces.count()) delete theCleanIfaces.pop();
 }
 
 // hosts are all host addresses; agents are unique agent addresses
@@ -538,7 +537,6 @@ int PolyApp::makeAddresses(int hidx, AddrSyms &agents, int agentsPerHost) {
 
 		const String &ifName = agents[i]->ifName();
 		const NetAddr &addr = agents[i]->val();
-		deleteAddresses(ifName);
 
 		InAddress netmask;
 		if (addr.addrN().family() == AF_INET6) {
@@ -559,7 +557,10 @@ int PolyApp::makeAddresses(int hidx, AddrSyms &agents, int agentsPerHost) {
 		int &aliasIndex = result.first->second;
 		if (result.second) {
 			// insertion took place, iface is seen for the first time
-			aliasIndex = iface.nextAliasIndex();
+			if (TheOpts.deleteOldAliases)
+				deleteAddresses(ifName);
+			else
+				aliasIndex = iface.nextAliasIndex();
 		}
 		if (!Should(iface.addAlias(addr.addrN(), aliasIndex++, netmask))) {
 			agents[i]->print(cerr << "error: " << ifName
@@ -573,17 +574,8 @@ int PolyApp::makeAddresses(int hidx, AddrSyms &agents, int agentsPerHost) {
 	return aliasCount;
 }
 
-// delete old aliases if needed
+// delete old aliases
 void PolyApp::deleteAddresses(const String &ifname) {
-	if (!TheOpts.deleteOldAliases)
-		return;
-
-	// check if we have done that already
-	for (int i = 0; i < theCleanIfaces.count(); ++i) {
-		if (*theCleanIfaces[i] == ifname)
-			return;
-	}
-
 	NetIface iface;
 	iface.name(ifname);
 	const int delCount = iface.delAliases();
@@ -594,24 +586,23 @@ void PolyApp::deleteAddresses(const String &ifname) {
 		Comment << "error: " << ifname 
 			<< ": failed to delete old IP aliases" << endc;
 	}
-
-	// remember that we tried to clean this iface
-	theCleanIfaces.append(new String(ifname));
 }
 
-void PolyApp::addUniqueAddrs(AddrSyms *const addrs, const NetAddrSym &s, HostMap &seen, ostream &skipped) const {
+void PolyApp::addUniqueAddrs(AddrSyms *const addrs, const NetAddrSym &s, HostMap &seen, Array<NetAddr> &skipped, HostMap &skipped_map) const {
+	const NetAddr addr(s.val().addrN(), -1); // we only care about the IP
+	int idx = -1;
 	int subnet;
 	if ((s.subnet(subnet) || s.val().addrN().family() == AF_INET6) &&
 		s.ifName()) {
-		const NetAddr addr(s.val().addrN(), -1); // we only care about the IP
-		int idx = -1;
 		if (!seen.find(addr, idx)) {
 			if (addrs)
 				addrs->append(&SymCast(NetAddrSym, *s.clone()));
 			(void)seen.addAt(idx, addr);
                 }
-	} else if (addrs)
-		s.print(skipped << ' ', "");
+	} else if (addrs && !skipped_map.find(addr, idx)) {
+		skipped_map.addAt(idx, addr);
+		skipped.append(addr);
+	}
 }
 
 void PolyApp::getHostAddrs(Array<NetAddr*> &hosts) const {
@@ -623,7 +614,8 @@ void PolyApp::getHostAddrs(Array<NetAddr*> &hosts) const {
 // hosts when IPs are sliced: a,b,a,b must yield a|b IPs for a,a|b,b agents 
 // Skip addresses without ifnames: addrs are used for alias creation only
 void PolyApp::getAgentAliasAddrs(AddrSyms &addrs) const {
-	ostringstream skipped;
+	Array<NetAddr> skipped;
+	std::auto_ptr<HostMap> skipped_map; // lookup table to weed out duplicates
 	Array<NetAddrSym *> bench_addrs;
 	std::auto_ptr<HostMap> bench_addrs_map; // lookup table to weed out duplicates
 
@@ -632,9 +624,12 @@ void PolyApp::getAgentAliasAddrs(AddrSyms &addrs) const {
 		PglStaticSemx::TheBench->side(sideName()) &&
 		PglStaticSemx::TheBench->side(sideName())->addresses(bench_addrs)) {
 		bench_addrs_map.reset(new HostMap(bench_addrs.count()));
+		skipped_map.reset(new HostMap(bench_addrs.count()));
 		// append unique IP addresses with ifnames only
-		for (int i = 0; i < bench_addrs.count(); ++i)
-			addUniqueAddrs(&addrs, *bench_addrs[i], *bench_addrs_map, skipped);
+		for (int i = 0; i < bench_addrs.count(); ++i) {
+			addUniqueAddrs(&addrs, *bench_addrs[i], *bench_addrs_map,
+				skipped, *skipped_map);
+		}
 	}
 
 	// count all addresses to size the lookup table
@@ -644,8 +639,13 @@ void PolyApp::getAgentAliasAddrs(AddrSyms &addrs) const {
 
 	// append unique IP addresses with ifnames only
 	HostMap agent_addrs_map(allCount); // lookup table to weed out duplicates
-	for (AgentAddrIter i(PglStaticSemx::TheAgentsToUse, theAgentType); i; ++i)
-		addUniqueAddrs(bench_addrs_map.get() ? 0 : &addrs, ConstSymCast(NetAddrSym, *i.addressSym()), agent_addrs_map, skipped);
+	if (!skipped_map.get())
+		skipped_map.reset(new HostMap(allCount));
+	for (AgentAddrIter i(PglStaticSemx::TheAgentsToUse, theAgentType); i; ++i) {
+		addUniqueAddrs(bench_addrs_map.get() ? 0 : &addrs,
+			ConstSymCast(NetAddrSym, *i.addressSym()),
+			agent_addrs_map, skipped, *skipped_map);
+	}
 
 	if (bench_addrs_map.get()) {
 		allCount = bench_addrs.count();
@@ -672,14 +672,27 @@ void PolyApp::getAgentAliasAddrs(AddrSyms &addrs) const {
 		}
 	}
 
-	if (skipped.tellp()) {
-		skipped << ends;
-		Comment(6) << "fyi: these " << theAgentType << " address(es) have " <<
-			"no interface names or netmasks: " << skipped.str() << endc;
-		streamFreeze(skipped, false);
+	if (addrs.empty()) {
+		Comment(6) << "fyi: all " << allCount << ' ' << theAgentType
+			<< " IP address(es) lack interface name or netmask" << endc;
+	} else if (!skipped.empty()) {
+		Comment(6) << "fyi: these " << theAgentType
+			<< " IP address(es) have no interface names or netmasks: ";
+		Array<IpRange> ranges;
+		CompactAllAddrs(skipped, ranges);
+		const int limit = 100;
+		for (int i = 0; i < Min(ranges.count(), limit); ++i) {
+			if (i)
+				Comment << ", ";
+			Comment << ranges[i];
+		}
+		if (ranges.count() > limit)
+			Comment << ", ...";
+		Comment << endc;
 	}
 	Comment(3) << "found " << allCount << ' ' << theAgentType <<
-		" IP addresses total (" << addrs.count() << " unique)." << endc;
+		" IP address(es) total (" << addrs.count() <<
+		" unique address(es) with interface name and netmask)." << endc;
 
 	// cleanup
 	while (bench_addrs.count()) delete bench_addrs.pop();
@@ -1014,10 +1027,6 @@ int PolyApp::run(int argc, char *argv[]) {
 	logGlobals(); // after makeAgents (hence, POST/PUT ctypes) are computed
 	describeLocals(); // after persistence (hence, agent IDs) is loaded
 
-	// free some memory
-	PglStaticSemx::Destroy();
-	thePglCfg = 0; 
-
 	flushState();
 
 	// flush logs headers
