@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -21,7 +21,8 @@ void OptGrp::share(OptGrp *other) {
 /* Opt */
 
 Opt::Opt(OptGrp *aGrp, const char *aName, const char *aDescr):
-	theGrp(aGrp), theCmdLine(0), theName(aName), theDescr(aDescr) {
+	theGrp(aGrp), theCmdLine(0), theName(aName), theDescr(aDescr),
+	isSet(false) {
 
 	if (const char *h = theName.chr(' ')) {
 		theType = theName(h+1 - theName.cstr(), theName.len());
@@ -36,3 +37,8 @@ Opt::Opt(OptGrp *aGrp, const char *aName, const char *aDescr):
 
 Opt::~Opt() {
 }
+
+const bool Opt::set(const String &name, const String &val) {
+	isSet = true;
+	return parse(name, val);
+}
