@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__BASE_OPT_H
@@ -35,20 +35,24 @@ class Opt {
 
 		void cmdLine(CmdLine *aCmdLine) { theCmdLine = aCmdLine; }
 
-		virtual bool parse(const String &name, const String &val) = 0;
+		const bool set(const String &name, const String &val);
 		virtual void report(ostream &) const = 0;
 		virtual bool visible() const { return true; } // use in reports
 
 		const String &name() const { return theName; }
 		const String &type() const { return theType; }
 		const String &descr() const { return theDescr; }
+		const bool wasSet() const { return isSet; }
 
 	protected:
+		virtual bool parse(const String &name, const String &val) = 0;
+
 		OptGrp *theGrp;
 		CmdLine *theCmdLine;
 		String theName;
 		String theType;
 		String theDescr;
+		bool isSet; // explicitly configured rather than relying on default
 };
 
 
