@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -13,7 +13,8 @@
 Progress TheProgress;
 
 
-Progress::Progress(): ssl("ssl", "SSL"), ftp("ftp", "FTP") {
+Progress::Progress():
+	socks("socks", "SOCKS"), ssl("ssl", "SSL"), ftp("ftp", "FTP") {
 	theXactCnt = theErrCnt = 0;
 }
 
