@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -21,14 +21,14 @@ CacheEntry::CacheEntry(const ObjId &anId): nextInIdx(0), theDistrPoint(0) {
 CacheEntry::~CacheEntry() {
 	// paranoid asserts to check that we cleaned the entry before
 	// destroying it
-	Assert(isolated());
+	Assert(cacheEntries.isolated());
 	Assert(!theDistrPoint);
 	Assert(!nextInIdx);
 }
 
 void CacheEntry::reset() {
 	Assert(!theDistrPoint);
-	QueueItem::isolate();
+	cacheEntries.isolate();
 	theDistrPoint = 0;
 	theId.reset();
 	theObjSize = -1;
