@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__CACHE_CACHEENTRY_H
@@ -14,7 +14,7 @@ class DistrPoint;
 
 // a central point for information about all objects handled by the cache
 
-class CacheEntry: public QueueItem {
+class CacheEntry {
 	public:
 		friend class DistrPoint;
 
@@ -28,8 +28,8 @@ class CacheEntry: public QueueItem {
 		const ObjId &id() const { return theId; }
 		DistrPoint *distrPoint() { return theDistrPoint; }
 		Size objSize() const { return theObjSize; }
-		bool cached() const { return !isolated(); }
-		bool stray() const { return isolated() && !theDistrPoint; }
+		bool cached() const { return !cacheEntries.isolated(); }
+		bool stray() const { return cacheEntries.isolated() && !theDistrPoint; }
 
 		DistrPoint *startDistributing(Cache *cache);
 		void stopDistributing(DistrPoint *dp);
@@ -38,6 +38,7 @@ class CacheEntry: public QueueItem {
 		void objSize(Size sz) { theObjSize = sz; }
 
 	public: /* XXX */
+		QueuePlace<CacheEntry> cacheEntries;
 		CacheEntry *nextInIdx; // next entry in cache index
 
 	protected:
