@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__CACHE_CACHEPOLICY_H
@@ -25,7 +25,7 @@ class CachePolicy {
 		CacheEntry *nextVictim() { return theQueue.firstOut(); }
 
 	protected:
-		Queue<CacheEntry> theQueue;
+		Queue<CacheEntry, &CacheEntry::cacheEntries> theQueue;
 		Cache *theCache;
 };
 
