@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -9,7 +9,6 @@
 #include "xstd/h/iostream.h"
 #include "xstd/h/iomanip.h"
 
-#include "xstd/BitMask.h"
 #include "xstd/Ssl.h"
 #include "base/ObjId.h"
 #include "base/BStream.h"
@@ -34,6 +33,7 @@
 #include "runtime/globals.h"
 #include "runtime/SslWrap.h"
 #include "runtime/SslWraps.h"
+#include "runtime/UserCred.h"
 #include "csm/BodyIter.h"
 #include "csm/ContentSel.h"
 #include "csm/ContentCfg.h"
@@ -48,7 +48,6 @@
 #include "client/IcpCltXact.h"
 #include "client/SessionMgr.h"
 #include "client/PrivCache.h"
-#include "client/UserCred.h"
 #include "client/CltCfg.h"
 #include "client/Client.h"
 
@@ -101,17 +100,20 @@ void Client::configure(const RobotSym *cfg, const NetAddr &aHost) {
 	if (cfg->privCache(pcCap))
 		thePrivCache = new PrivCache(pcCap);
 
-	theCfg->selectHttpProxy(theHttpProxyAddr); // sticky selection
-	if (!theCfg->selectFtpProxy(theFtpProxyAddr)) // sticky selection
-		theFtpProxyAddr = theHttpProxyAddr;
-
+	bool doSocksChaining = false;
+	theCfg->selectSocksProxy(theSocksProxyAddr, doSocksChaining); // sticky selection
+	if (!theSocksProxyAddr || doSocksChaining) {
+		theCfg->selectHttpProxy(theHttpProxyAddr); // sticky selection
+		if (!theCfg->selectFtpProxy(theFtpProxyAddr)) // sticky selection
+			theFtpProxyAddr = theHttpProxyAddr;
+	}
 	int srvCnt;
 	if (theFtpProxyAddr && theHttpProxyAddr)
 		srvCnt = theFtpProxyAddr == theHttpProxyAddr ? 1 : 2;
 	else
 		srvCnt = theCfg->viservCount();
 	theConnMgr = new CltConnMgr;
-	theConnMgr->configure(theSockOpt, cfg, srvCnt);
+	theConnMgr->configure(theSockOpt, this, srvCnt);
 	theConnMgr->portMgr(cfgPortMgr());
 	theConnMgr->idleTimeout(cfg->idlePconnTimeout());
 
@@ -229,17 +231,24 @@ void Client::becomeIdle() {
 
 void Client::describe(ostream &os) const {
 	Agent::describe(os);
-	if (theFtpProxyAddr || theHttpProxyAddr) {
-		os << " via";
+	if (theSocksProxyAddr || theFtpProxyAddr || theHttpProxyAddr) {
+		os << " via ";
+		if (theSocksProxyAddr) {
+			os << theSocksProxyAddr << " (socks)";
+			if (theHttpProxyAddr || theFtpProxyAddr)
+				os << ", ";
+		}
 		if (theHttpProxyAddr) {
-			os << ' ' << theHttpProxyAddr;
-			if (theFtpProxyAddr != theHttpProxyAddr)
-				os << " (http)";
-		}
-		if (theFtpProxyAddr && theFtpProxyAddr != theHttpProxyAddr) {
-			os << (theHttpProxyAddr ? ", " : " ") << theFtpProxyAddr <<
-				" (ftp)";
+			os << theHttpProxyAddr << " (http";
+			if (theFtpProxyAddr == theHttpProxyAddr)
+				os << " and ftp)";
+			else if (theFtpProxyAddr)
+				os << ") or ";
+			else
+				os << ')';
 		}
+		if (theFtpProxyAddr && theFtpProxyAddr != theHttpProxyAddr)
+			os << theFtpProxyAddr << " (ftp)";
 	}
 }
 
@@ -324,23 +333,26 @@ bool Client::launch(CltXact *x) {
 	if (!conn) {
 		Assert(!theConnMgr->atHardConnLimit());
 
-		const NetAddr &connAddr = x->nextHop();
-		Assert(connAddr);
-		Assert(!connAddr.isDomainName());
+		const NetAddr &nextHighHop = x->nextHighHop();
+		Assert(nextHighHop);
+
+		const NetAddr &nextTcpHop = x->nextTcpHop();
+		Assert(nextTcpHop);
+		Assert(!nextTcpHop.isDomainName());
 
 		const ProtoIntvlPtr protoStat = transport(x->oid()) == pFTP ?
 			&StatIntvlRec::theFtpStat : 0;
-		conn = theConnMgr->get(x->oid(), connAddr, protoStat);
+		conn = theConnMgr->get(x->oid(), nextHighHop, nextTcpHop, protoStat);
 
 		if (!conn && ReportError(errConnectEstb)) {
-			Comment(1) << theHost << " failed to connect to " << connAddr <<
-			endc;
+			Comment(1) << theHost << " failed to connect to "
+				<< nextTcpHop << endc;
 		}
 	}
 
 	if (conn) {
 		theCcXactLvl++;
-		x->exec(this, conn);
+		x->exec(conn);
 		return true;
 	}
 
@@ -367,10 +379,10 @@ bool Client::tryLaunch(CltXact *x) {
 	if (isIdle)
 		return launchCanceled(x);
 
-	if (!x->nextHop()) {
+	if (!x->nextHighHop()) {
 
 		// should we ask peers for the best server?
-		// XXX: remove askedPeer; the x->nextHop() should be enough?
+		// XXX: remove askedPeer; the x->nextHighHop() should be enough?
 		if (theCfg->thePeerIcp && !x->askedPeer()) {
 			askPeer(theCfg->thePeerIcp, x);
 			return true;
@@ -382,15 +394,17 @@ bool Client::tryLaunch(CltXact *x) {
 			return false;
 
 		// should we lookup the next hop address?
-		if (theDnsMgr->needsLookup(x->nextHop())) {
+		if (theDnsMgr->needsLookup(x->nextTcpHop())) {
 			// async call unless fails immediately
-			if (lookupAddr(x))
+			if (theDnsMgr->lookup(x->nextTcpHop(), x))
 				return true;
 			launchFailed(x);
 			return false;
 		}
 	}
 
+	x->lifeTimeLimit(selectLifetime());
+
 	// check if we should postpone the xaction
 	if (!x->conn() && theConnMgr->atHardConnLimit())
 		return suspendXact(x);
@@ -400,7 +414,7 @@ bool Client::tryLaunch(CltXact *x) {
 
 bool Client::suspendXact(CltXact *x) {
 	if (theCfg->theWaitXactLmt < 0 || theLaunchDebts.count() < theCfg->theWaitXactLmt) {
-		theLaunchDebts.append(x);
+		theLaunchDebts.enqueue(x);
 		Broadcast(TheWaitBegChannel, x);
 		return true;
 	}
@@ -418,8 +432,12 @@ void Client::resumeXact() {
 	launch(dequeSuspXact());
 }
 
-CltXact *Client::dequeSuspXact() {
-	CltXact *x = theLaunchDebts.pop();
+// called with nil x unless called by a lifetime-expired xaction
+CltXact *Client::dequeSuspXact(CltXact *x) {
+	Must(!theLaunchDebts.empty());
+	if (!x)
+		x = theLaunchDebts.firstOut();
+	theLaunchDebts.dequeue(x);
 	Broadcast(TheWaitEndChannel, x);
 	return x;
 }
@@ -453,6 +471,7 @@ void Client::recycleXact(CltXact *x) {
 CltXact *Client::genXact(const ObjId &oid, CltXact *cause) {
 	CltXact *x = getXact(oid);
 	x->oid(oid);
+	x->owner(this);
 	if (cause) {
 		cause->noteChildNew(x);
 		x->cause(cause);
@@ -553,8 +572,11 @@ void Client::selectScheme(ObjId &oid) {
 
 void Client::selectReqType(ObjId &oid) {
 	static RndGen rng;
-	int reqType = rng.event(TheStatPhaseMgr->specialMsgFactor().current()) ?
-		(int)theCfg->theReqTypeSel->trial() : rqtBasic;
+	int reqType = rqtBasic;
+	if (rng.event(TheStatPhaseMgr->specialMsgFactor().current())) {
+		const CltCfg &cfg = behavior(oid, &CltBehaviorCfg::haveReqTypes);
+		reqType = cfg.theReqTypeSel->ltrial();
+	}
 
 	// force basic request type for all FTP requests except upload
 	if (oid.scheme() == Agent::pFTP &&
@@ -572,13 +594,16 @@ void Client::selectReqType(ObjId &oid) {
 
 void Client::selectReqMethod(ObjId &oid) {
 	static RndGen rng;
-	int reqMethod;
+	int reqMethod = rqmGet;
 	if (oid.upload())
 		reqMethod = rqmPut;
-	else
-		reqMethod = oid.scheme() == Agent::pHTTP &&
-			rng.event(TheStatPhaseMgr->specialMsgFactor().current()) ?
-			(int)theCfg->theReqMethodSel->trial() : rqmGet;
+	else if (oid.scheme() == Agent::pHTTP &&
+		rng.event(TheStatPhaseMgr->specialMsgFactor().current())) {
+		const CltCfg &cfg =
+			behavior(oid, &CltBehaviorCfg::haveReqMethods);
+		reqMethod = cfg.theReqMethodSel->ltrial();
+	}
+
 	oid.get(reqMethod == rqmGet);
 	oid.post(reqMethod == rqmPost);
 	oid.head(reqMethod == rqmHead);
@@ -655,6 +680,14 @@ bool Client::credentialsFor(ObjId &oid, UserCred &cred) const {
 	return false;
 }
 
+bool Client::credentialsFor(const Connection &conn, UserCred &cred) const {
+	if (genCredentials(cred)) {
+		cred.finalize(conn);
+		return true;
+	}
+	return false;
+}
+
 bool Client::genCredentials(UserCred &cred) const {
 	cred = UserCred(theCredentials);
 	// invalidate some credentials, if configured
@@ -726,11 +759,14 @@ void Client::noteEmbedded(CltXact *parent, const ObjId &oid) {
 	child->page(parent->page());
 
 	// simulate cached DNS responses for embedded objects
-	if (!oid.foreignUrl() && oid.viserv() == parent->oid().viserv())
-		child->nextHopVar() = parent->nextHop();
+	if (!oid.foreignUrl() && oid.viserv() == parent->oid().viserv()) {
+		child->nextHighHop(parent->nextHighHop());
+		child->nextTcpHop(parent->nextTcpHop());
+	}
 
 	// pipeline if possible
-	if (child->nextHop() && child->nextHop() == parent->nextHop()) {
+	if (child->nextHighHop() &&
+		child->nextHighHop() == parent->nextHighHop()) {
 		if (PipelinedCxm *mgr = parent->getPipeline())
 			child->pipeline(mgr);
 	}
@@ -757,16 +793,14 @@ void Client::notePeerAsked(IcpCltXact *q) {
 	tryLaunch(x);
 }
 
-bool Client::lookupAddr(CltXact *x) {
-	return theDnsMgr->lookup(x->nextHop(), x);
-}
-
 void Client::noteAddrLookup(const NetAddr &addr, CltXact *x) {
 	Assert(x);
 
 	// successful lookup?
 	if (addr) {
-		x->nextHopVar() = addr; // update
+		if (x->nextTcpHop() == x->nextHighHop())
+			x->nextHighHop(addr);
+		x->nextTcpHop(addr);
 		tryLaunch(x);
 	} else {
 		launchFailed(x);
@@ -775,15 +809,16 @@ void Client::noteAddrLookup(const NetAddr &addr, CltXact *x) {
 
 // select the address for the next hop connection
 bool Client::setNextHopAddr(CltXact *x) const {
-	NetAddr &addr = x->nextHopVar();
+	if (theSocksProxyAddr)
+		x->nextTcpHop(theSocksProxyAddr);
 
+	NetAddr addr;
 	if (x->usePeer() && theCfg->thePeerHttp)
-		return addr = theCfg->thePeerHttp;
-
-	addr = proxy(x->oid());
-	if (addr)
-		return true;
-
+		addr = theCfg->thePeerHttp;
+	else
+	if (proxy(x->oid()))
+		addr = proxy(x->oid());
+	else
 	if (x->oid().foreignUrl()) {
 		const char *uri = x->oid().foreignUrl().cstr();
 		if (!SkipHostInUri(uri, uri+x->oid().foreignUrl().len(), addr)) {
@@ -791,10 +826,12 @@ bool Client::setNextHopAddr(CltXact *x) const {
 				Comment << "foreign URL: " << uri << endc;
 			return false;
 		}
-		return addr;
 	}
+	else
+		addr = Oid2UrlHost(x->oid());
 
-	return addr = Oid2UrlHost(x->oid());
+	x->nextHighHop(addr);
+	return addr;
 }
 
 // fast, used by transactions that connect w/o theConnMgr help
@@ -977,6 +1014,19 @@ void Client::absorbCookies(const ObjId &oid, HttpCookies *&c) {
 	c = 0;
 }
 
+const CltCfg &Client::behavior(const ObjId &oid, const CltBehaviorCfg::Predicate predicate) const {
+	Assert(oid.type() >= 0);
+	const CltBehaviorCfg *const behaviorCfg =
+		TheCltBehaviorCfgs.get(oid.type());
+	return *(behaviorCfg && (behaviorCfg->*predicate)() ? behaviorCfg :
+		theCfg);
+}
+
+RangeCfg::RangesInfo Client::makeRangeSet(ostream &os, const ObjId &oid, ContentCfg &contentCfg) const {
+	const CltCfg &cfg = behavior(oid, &CltBehaviorCfg::haveRanges);
+	return cfg.makeRangeSet(os, oid, contentCfg);
+}
+
 const Client::HostInfo *Client::hostInfo(const ObjId &oid) const {
 	const HostInfo *hostInfo(0);
 	if (oid.foreignUrl()) {
