@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__CLIENT_CLIENT_H
@@ -8,7 +8,9 @@
 
 #include <map>
 
+#include "client/CltBehaviorCfg.h"
 #include "client/CltCfg.h"
+#include "client/CltXact.h"
 #include "client/MembershipMap.h"
 #include "runtime/httpHdrs.h"
 #include "runtime/Agent.h"
@@ -16,9 +18,7 @@
 #include "runtime/XactFarm.h"
 #include "xstd/AlarmClock.h"
 
-class BitMask;
 class CltConnMgr;
-class CltXact;
 class ContentCfg;
 class DistrPoint;
 class DnsMgr;
@@ -52,6 +52,8 @@ class Client: public Agent, public AlarmUser, public BcastRcver {
 		CltXact *fetch(const ObjId &oid, DistrPoint *dp);
 
 		virtual CltCfg *cfg() { return theCfg; }
+		const CltCfg *cfg() const { return theCfg; }
+		CltConnMgr *connMgr() { return theConnMgr; }
 		const Memberships &memberships() const { return theMemberships; }
 
 		bool hasCredentials() const;
@@ -84,17 +86,23 @@ class Client: public Agent, public AlarmUser, public BcastRcver {
 		HttpAuthScheme proxyAuthScheme(const ObjId &oid) const;
 		HttpAuthScheme originAuthScheme(const ObjId &oid) const;
 		bool credentialsFor(ObjId &, UserCred &) const;
+		bool credentialsFor(const Connection &, UserCred &) const;
 		void noteProxyAuthReq(CltXact *, HttpAuthScheme);
 		void noteOriginAuthReq(CltXact *, HttpAuthScheme);
 		void notePeerAsked(IcpCltXact *);
 		IcpClient *icpClient();
 
+		const NetAddr &socksProxy() const { return theSocksProxyAddr; }
 		const NetAddr &proxy(const ObjId &) const;
 		HttpCookies *cookies(const ObjId &) const;
 		void absorbCookies(const ObjId &, HttpCookies *&);
+		const CltCfg &behavior(const ObjId &oid, const CltBehaviorCfg::Predicate predicate) const;
+		RangeCfg::RangesInfo makeRangeSet(ostream &os, const ObjId &oid, ContentCfg &contentCfg) const;
 
 		PortMgr *portMgr();
 
+		CltXact *dequeSuspXact(CltXact *x = 0);
+
 	protected:
 		PortMgr *cfgPortMgr();
 
@@ -104,7 +112,6 @@ class Client: public Agent, public AlarmUser, public BcastRcver {
 		bool tryLaunch(CltXact *x);
 		bool suspendXact(CltXact *x);
 		void resumeXact();
-		CltXact *dequeSuspXact();
 		virtual void loneXactFollowup();
 
 		void recycleXact(CltXact *x);
@@ -121,8 +128,6 @@ class Client: public Agent, public AlarmUser, public BcastRcver {
 		void selectReqType(ObjId &oid);
 		void selectReqMethod(ObjId &oid);
 
-		bool lookupAddr(CltXact *x);
-
 		virtual void scheduleLaunch(Time lastLaunch) = 0;
 		virtual bool launchCanceled(CltXact *x) = 0; // with body
 		bool launchFailed(CltXact *x);
@@ -136,7 +141,6 @@ class Client: public Agent, public AlarmUser, public BcastRcver {
 		};
 		void configureReqIds(Array<ReqId> &ids, const Array<ContentCfg*> &cfgs);
 
-		bool askNextHop(CltXact *x);
 		bool setNextHopAddr(CltXact *x) const;
 
 		CltXact *getXact(const ObjId &oid);
@@ -178,13 +182,14 @@ class Client: public Agent, public AlarmUser, public BcastRcver {
 
 		PrivCache *thePrivCache;        // private cache for this agent
 
-		Array<CltXact*> theLaunchDebts; // postponed requests
+		Queue<CltXact, &CltXact::waitingXacts> theLaunchDebts; // postponed requests
 
 		CltCfg *theCfg;         // client configuration (maybe shared)
 		CltConnMgr *theConnMgr; // one for all servers
 		DnsMgr *theDnsMgr;      // DNS client, cache, etc.
 		SessionMgr *theSessionMgr; // manages busy/idle periods
 
+		NetAddr theSocksProxyAddr;
 		NetAddr theFtpProxyAddr;
 		NetAddr theHttpProxyAddr;
 
