@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -9,7 +9,7 @@
 #include "xstd/h/iomanip.h"
 
 #include "xstd/rndDistrs.h"
-#include "xstd/StrIdentifier.h"
+#include "xstd/StringIdentifier.h"
 #include "xstd/Ssl.h"
 #include "base/RndPermut.h"
 #include "base/OidGenStat.h"
@@ -31,6 +31,7 @@
 #include "runtime/httpHdrs.h"
 #include "csm/XmlTagIdentifier.h"
 #include "csm/ContentMgr.h"
+#include "csm/ContentSel.h"
 #include "client/WarmupPlan.h"
 #include "client/ServerRep.h"
 #include "client/RegExGroup.h"
@@ -48,7 +49,7 @@ CltCfg::CltCfg(): theRobot(0),
 	theOriginSel(0), theReqTypeSel(0), theReqMethodSel(0),
 	thePopModel(0),
 	theBusyPeriod(0), theIdlePeriodDur(0),
-	theFtpProxyCycleCnt(0), theHttpProxyCycleCnt(0),
+	theFtpProxyCycleCnt(0), theHttpProxyCycleCnt(0), theSocksProxyCycleCnt(0),
 	thePipelineDepth(0),
 	theRecurRatio(-1), theSpnegoAuthRatio(-1), theEmbedRecurRatio(-1),
 	theAbortProb(0), theAuthError(0),
@@ -63,7 +64,7 @@ CltCfg::CltCfg(): theRobot(0),
 	thePostContentSel(0), theUploadContentSel(0),
 	theForeignInterestProb(-1),
 	thePublicInterestProb(-1),
-	thePassiveFtp(-1),
+	thePassiveFtp(-1), theSocksProb(-1), theSocksChainingProb(-1),
 	genUniqUrls(false), didWarmup(false), theRangeSel(0),
 	ftpProxiesSet(false) {
 }
@@ -75,6 +76,8 @@ CltCfg::~CltCfg() {
 		delete theFtpProxies.dequeue();
 	while (theHttpProxies.count())
 		delete theHttpProxies.dequeue();
+	while (theSocksProxies.count())
+		delete theSocksProxies.dequeue();
 	delete theOriginSel;
 	delete theReqTypeSel;
 	delete theReqMethodSel;
@@ -83,7 +86,6 @@ CltCfg::~CltCfg() {
 	delete theContainerTags;
 	delete theAcceptedContentCodings;
 	delete theForeignWorld;
-	delete theAcceptedContentCodings;
 	delete theWarmupPlan;
 	delete thePostContentSel;
 	delete theUploadContentSel;
@@ -166,6 +168,8 @@ void CltCfg::configure(const RobotSym *aRobot) {
 			<< endl << xexit;
 
 	theRobot->passiveFtp(thePassiveFtp);
+
+	checkConfiguration();
 }
 
 int CltCfg::viservLimit() const {
@@ -187,9 +191,11 @@ void CltCfg::configureInterests() {
                         else
 			if (istrs[i]->val() == "private")
 				privProb = iprobs[i];
-			else
-				cerr << theRobot->loc() << "interests array can have only \"foreign\", \"private\" and \"public\" values" 
-				<< theRobot->kind() << endl << xexit;
+			else {
+				cerr << theRobot->loc() << "interests array can "
+					"have only \"foreign\", \"private\" and "
+					"\"public\" values" << endl << xexit;
+			}
 		if (privProb > 0) {
 			thePublicInterestProb = pubProb / (privProb + pubProb);
 			Comment(1) << theRobot->loc() << "private Robot URLs are "
@@ -204,7 +210,7 @@ void CltCfg::configureInterests() {
 }
 
 void CltCfg::configureReqTypes() {
-	static StrIdentifier sidf;
+	static StringIdentifier sidf;
 	if (!sidf.count()) {
 		sidf.add("Basic", Client::rqtBasic);
 		sidf.add("Ims200", Client::rqtIms200);
@@ -212,7 +218,6 @@ void CltCfg::configureReqTypes() {
 		sidf.add("Reload", Client::rqtReload);
 		sidf.add("Range", Client::rqtRange);
 		sidf.add("Upload", Client::rqtUpload);
-		sidf.optimize();
 	}
 
 	theReqTypeSel = theRobot->msgTypes(sidf);
@@ -228,13 +233,12 @@ void CltCfg::configurePipeDepth() {
 }
 
 void CltCfg::configureReqMethods() {
-	static StrIdentifier sidf;
+	static StringIdentifier sidf;
 	if (!sidf.count()) {
 		sidf.add("GET", Client::rqmGet);
 		sidf.add("HEAD", Client::rqmHead);
 		sidf.add("POST", Client::rqmPost);
 		sidf.add("PUT", Client::rqmPut);
-		sidf.optimize();
 	}
 
 	theReqMethodSel = theRobot->reqMethods(sidf);
@@ -248,9 +252,20 @@ void CltCfg::configureOrigins() {
 
 	RndDistr *iad = 0;
 	if (!theRobot->origins(origNames, theOriginSel) || !origNames.count()) {
-		if (!theRobot->reqInterArrival(iad) || iad)
-			cerr << theRobot->loc() << "no origin addresses specified for active robot " 
-				<< theRobot->kind() << endl << xexit;
+		const bool isActive = !theRobot->reqInterArrival(iad) || iad;
+		if (isActive && theForeignInterestProb < 1)
+			Comment(0) << theRobot->loc() << "no origin addresses "
+				"specified for active Robot '" <<
+				theRobot->kind() << "' with some non-foreign "
+				"interest" << endc << xexit;
+		return;
+	}
+	if (theForeignInterestProb >= 1) {
+		Comment(0) << theRobot->loc() << "origin addresses specified "
+			"for Robot '" << theRobot->kind() << "' with "
+			"foreign-only interest" << endc;
+		delete theOriginSel;
+		theOriginSel = 0;
 		return;
 	}
 
@@ -314,6 +329,160 @@ void CltCfg::checkTargets(int viserv) {
 	}
 }
 
+void CltCfg::checkConfiguration() const {
+	// collect reachable contents
+	ContentSet contents;
+	for (int i = 0; i < theViservs.count(); ++i) {
+		const NetAddr &visName = TheHostMap->at(theViservs[i])->theAddr;
+		int niamIdx; // name in AddrMap index
+		Assert(TheAddrMap->find(visName, niamIdx));
+		for (AddrMapAddrIter j = TheAddrMap->addrIter(niamIdx); j; ++j) {
+			int targetIdx = -1;
+			Assert(TheHostMap->find(j.addr(), targetIdx));
+			const HostCfg *const hcfg = TheHostMap->at(targetIdx);
+			Assert(hcfg && hcfg->theContent);
+			const Array<ContentCfg*> &srvContents =
+				hcfg->theContent->contents();
+			for (int k = 0; k < srvContents.count(); ++k)
+				contents.insert(srvContents[k]);
+		}
+	}
+
+	checkPostContentsConfiguration(contents);
+	checkUploadContentsConfiguration(contents);
+	checkRangesConfiguration(contents);
+}
+
+void CltCfg::checkPostContentsConfiguration(const ContentSet &contents) const {
+	const bool havePostContents = thePostContentSel;
+	if (havePostRequest()) {
+		if (!havePostContents) {
+			Comment(0) << theRobot->loc() << "'POST' request method"
+				" configured but no 'post_contents' set"
+				<< endc << xexit;
+		}
+		return;
+	}
+	bool noPostRequest = true;
+	for (ContentSet::const_iterator i = contents.begin();
+		i != contents.end(); ++i) {
+		const ContentCfg *const content = *i;
+		Assert(content);
+		const CltCfg *const behavior =
+			TheCltBehaviorCfgs.get(content->id());
+		if (!behavior || !behavior->havePostRequest())
+			continue;
+		if (!havePostContents) {
+			Comment(0) << behavior->theRobot->loc() << "'POST' "
+				"request method configured for Content '" <<
+				content->kind() << "' but Robot '" <<
+				theRobot->kind() << "' has no 'post_contents' "
+				"set" << endc << xexit;
+		}
+		noPostRequest = false;
+	}
+	if (havePostContents && noPostRequest) {
+		Comment(0) << theRobot->loc() << "'post_contents' set but no "
+			"'POST' request method configured" << endl << xexit;
+	}
+}
+
+void CltCfg::checkUploadContentsConfiguration(const ContentSet &contents) const {
+	const bool haveUploadContents = theUploadContentSel;
+	if (haveUploadRequest()) {
+		if (!haveUploadContents) {
+			Comment(0) << theRobot->loc() << "'Upload' request "
+				"type or 'PUT' request method configured but no"
+				" 'upload_contents' set" << endl << xexit;
+		}
+		return;
+	}
+	bool noUploadRequest = true;
+	for (ContentSet::const_iterator i = contents.begin();
+		i != contents.end(); ++i) {
+		const ContentCfg *const content = *i;
+		Assert(content);
+		const CltCfg *const behavior =
+			TheCltBehaviorCfgs.get(content->id());
+		if (!behavior || !behavior->haveUploadRequest())
+			continue;
+		if (!haveUploadContents) {
+			Comment(0) << behavior->theRobot->loc() << "'Upload' "
+				"request type or 'PUT' request method "
+				"configured for Content '" << content->kind()
+				<< "' but Robot '" << theRobot->kind() << "' "
+				"has no 'upload_contents' set" << endc << xexit;
+		}
+		noUploadRequest = false;
+	}
+	if (haveUploadContents && noUploadRequest) {
+		Comment(0) << theRobot->loc() << "'upload_contents' set but no "
+			"'Upload' request type or 'PUT' request method "
+			"configured" << endl << xexit;
+	}
+}
+
+void CltCfg::checkRangesConfiguration(const ContentSet &contents) const {
+	const bool haveRanges = theRangeSel;
+	if (haveRangeRequest() && haveRanges)
+		return;
+	bool noRangeRequest = true;
+	bool allRanges = true;
+	for (ContentSet::const_iterator i = contents.begin();
+		i != contents.end(); ++i) {
+		const ContentCfg *const content = *i;
+		Assert(content);
+		const CltCfg *const behavior =
+			TheCltBehaviorCfgs.get(content->id());
+		if (!behavior) {
+			allRanges = false;
+			continue;
+		}
+		const bool needRanges = behavior->theReqTypeSel ?
+			behavior->haveRangeRequest() : haveRangeRequest();
+		if (needRanges && !behavior->theRangeSel)
+			allRanges = false;
+		if (behavior->theRangeSel && !needRanges) {
+			Comment(0) << behavior->theRobot->loc() <<
+				"'ranges' set but no 'Range' request type "
+				"configured for Content '" << content->kind() <<
+				"' and Robot '" << theRobot->kind() << '\''
+				<< endl << xexit;
+		}
+		if (!haveRanges && !behavior->theRangeSel && needRanges) {
+			Comment(0) << behavior->theRobot->loc() << "'Range' "
+				"request type configured but no 'ranges' set "
+				"for Content '" << content->kind() << "' and "
+				"Robot '" << theRobot->kind() << '\''
+				<< endc << xexit;
+		}
+		if (behavior->haveRangeRequest())
+			noRangeRequest = false;
+	}
+	if (haveRanges && noRangeRequest) {
+		Comment(0) << theRobot->loc() << "'ranges' set but no 'Range' "
+			"request type configured" << endl << xexit;
+	}
+	if (haveRangeRequest() && !allRanges) {
+		Comment(0) << theRobot->loc() << "'Range' request "
+			"type configured but no 'ranges' set"
+			<< endl << xexit;
+	}
+}
+
+bool CltCfg::havePostRequest() const {
+	return theRobot->arrayHasElem("req_methods", "POST");
+}
+
+bool CltCfg::haveUploadRequest() const {
+	return theRobot->arrayHasElem("req_types", "Upload") ||
+		theRobot->arrayHasElem("req_methods", "PUT");
+}
+
+bool CltCfg::haveRangeRequest() const {
+	return theRobot->arrayHasElem("req_types", "Range");
+}
+
 void CltCfg::configureProxies() {
 	Array<NetAddr*> addrs;
 
@@ -347,6 +516,42 @@ void CltCfg::configureProxies() {
 		while (addrs.count())
 			theHttpProxies.enqueue(addrs.pop());
 	}
+
+	configureSocksProxies();
+}
+
+void CltCfg::configureSocksProxies() {
+	Array<NetAddr*> addrs;
+	const bool socksProbDefault = !theRobot->socksProb(theSocksProb);
+	if (theRobot->socksProxies(addrs)) {
+		theSocksProxies.resize(addrs.count());
+		while (addrs.count())
+			theSocksProxies.enqueue(addrs.pop());
+		if (socksProbDefault)
+			theSocksProb = 1;
+	} else if (theSocksProb > 0) {
+		cerr << theRobot->loc() << "Robot::socks_prob field "
+			<< "is positive but no Robot::socks_proxies configured"
+			<< endl << xexit;
+	}
+	const bool socksProxyPresent = !theSocksProxies.empty();
+	const bool otherProxyPresent =
+		!theHttpProxies.empty() || !theFtpProxies.empty();
+	theRobot->socksChainingProb(theSocksChainingProb);
+	if (theSocksChainingProb > 0) {
+		if (!socksProxyPresent) {
+			cerr << theRobot->loc()
+				<< "Robot::socks_chaining_prob field is positive "
+				<< "but no Robot::socks_proxies configured"
+				<< endl << xexit;
+		}
+		if (!otherProxyPresent) {
+			cerr << theRobot->loc()
+				<< "Robot::socks_chaining_prob field is positive "
+				<< "but no Robot::http_proxies/Robot::ftp_proxies "
+				<< "configured"	<< endl << xexit;
+		}
+	}
 }
 
 void CltCfg::configureCredentials() {
@@ -426,37 +631,8 @@ void CltCfg::configureTrace() {
 
 void CltCfg::configureRanges() {
 	Array<const RangeSym*> syms;
-	bool range_req_type = false;
-
-	{
-		// check if "Range" request type configured
-		Array<StringSym*> types;
-		Array<double> tprobs;
-		if (theRobot->msgTypes(types, tprobs))
-			for (int i = 0; i < types.count(); ++i)
-				if (types[i]->val() == "Range" &&
-					tprobs[i] > 0) {
-					range_req_type = true;
-					break;
-				}
-	}
-
-	if (!theRobot->ranges(syms, theRangeSel)) {
-		if (range_req_type)
-			cerr
-				<< theRobot->loc()
-				<< "Range request type configured "
-				<< "but no ranges set"
-				<< endl << xexit;
-
+	if (!theRobot->ranges(syms, theRangeSel))
 		return; // no ranges configured
-	}
-
-	if (!range_req_type)
-		cerr
-			<< theRobot->loc()
-			<< "Ranges set but no range request type configured"
-			<< endl << xexit;
 
 	theRangeSel->rndGen(LclRndGen("client_ranges"));
 	theRanges.stretch(syms.count());
@@ -482,20 +658,9 @@ void CltCfg::configureRanges() {
 }
 
 void CltCfg::configurePostContents() {
-	const bool post_method(theRobot->arrayHasElem("req_methods", "POST"));
 	Array<ContentSym*> syms;
-	if (!theRobot->reqContents("post_contents", syms, thePostContentSel)) {
-		if (post_method)
-			Comment << theRobot->loc() << "'POST' request method "
-				"configured but no 'post_contents' set"
-				<< endl << xexit;
-
+	if (!theRobot->reqContents("post_contents", syms, thePostContentSel))
 		return; // no contents configured
-	}
-
-	if (!post_method)
-		Comment << theRobot->loc() << "'post_contents' set but no "
-			"'POST' request method configured" << endl << xexit;
 
 	thePostContentSel->rndGen(LclRndGen("client_post_contents"));
 	thePostContents.stretch(syms.count());
@@ -503,29 +668,17 @@ void CltCfg::configurePostContents() {
 }
 
 void CltCfg::configureUploadContents() {
-	const bool upload_request(theRobot->arrayHasElem("req_types", "Upload"));
-	const bool put_method(theRobot->arrayHasElem("req_methods", "PUT"));
 	Array<ContentSym*> syms;
 	if (!theRobot->reqContents("upload_contents", syms, theUploadContentSel)) {
-		if (!theRobot->reqContents("put_contents", syms, theUploadContentSel)) {
-			if (upload_request || put_method)
-				Comment << theRobot->loc() << "'Upload' request "
-					"type or 'PUT' request method configured "
-					"but no 'upload_contents' set"
-					<< endl << xexit;
-
+		if (!theRobot->reqContents("put_contents", syms, theUploadContentSel))
 			return; // no contents configured
-		} else
+		else {
 			Comment(1) << theRobot->loc() << "Robot::put_contents field "
 				"is deprecated in favor of the "
 				"Robot::upload_contents field" << endc;
+		}
 	}
 
-	if (!upload_request && !put_method)
-		Comment << theRobot->loc() << "'upload_contents' set but no "
-			"'Upload' request type or 'PUT' request method "
-			"configured" << endl << xexit;
-
 	theUploadContentSel->rndGen(LclRndGen("client_upload_contents"));
 	theUploadContents.stretch(syms.count());
 	TheContentMgr.get(syms, theUploadContents);
@@ -587,6 +740,23 @@ bool CltCfg::selectHttpProxy(NetAddr &newAddr) {
 	return true;
 }
 
+bool CltCfg::selectSocksProxy(NetAddr &newAddr, bool &doSocksChaining) {
+	if (theSocksProb <= 0)
+		return false;
+
+	static RndGen *const rng1 = LclRndGen("socks_prob");
+	if (!rng1->event(theSocksProb))
+		return false;
+
+	selectProxy(theSocksProxies, theSocksProxyCycleCnt, newAddr);
+	if (theSocksChainingProb > 0) {
+		static RndGen *const rng2 = LclRndGen("socks_chaining_prob");
+		doSocksChaining = rng2->event(theSocksChainingProb);
+	} else
+		doSocksChaining = false;
+	return true;
+}
+
 // XXX: merge this with SrvCfg::selectAbortCoord()
 void CltCfg::selectAbortCoord(XactAbortCoord &coord) {
 	static RndGen rng1, rng2; // uncorrelated unless theAbortProb is 1
@@ -630,19 +800,30 @@ bool CltCfg::followAllUris(const RepHdr &rep) const {
 	}
 }
 
-RangeCfg::RangesInfo CltCfg::makeRangeSet(ostream &os, const ObjId &oid, ContentCfg &contentCfg) {
+RangeCfg::RangesInfo CltCfg::makeRangeSet(ostream &os, const ObjId &oid, ContentCfg &contentCfg) const {
 	Assert(theRangeSel);
 	return theRanges[theRangeSel->ltrial()]->makeRangeSet(os, oid, contentCfg);
 }
 
 // may be called multiple times
 void CltCfg::startWarmup() {
-	if (!theWarmupPlan && !didWarmup)
+	if (!theWarmupPlan && !didWarmup && theForeignInterestProb < 1) {
 		theWarmupPlan = new WarmupPlan(theViservs);
+
+		Comment(7) << "started a warmup plan for Robot '" <<
+			theRobot->kind() << "' with " <<
+			theWarmupPlan->planCount() << " cold servers, out "
+			"of " << PubWorld::Count() << " already visible" << endc;
+	}
 }
 
 void CltCfg::stopWarmup() {
 	Assert(theWarmupPlan && !didWarmup);
+
+	Comment(7) << "stopped warmup plan for Robot '" << theRobot->kind() <<
+		"' with " << theWarmupPlan->planCount() << " cold servers, "
+		"out of " << PubWorld::Count() << " globally visible" << endc;
+
 	delete theWarmupPlan;
 	theWarmupPlan = 0;
 	didWarmup = true;
