@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -18,10 +18,11 @@
 #include "runtime/HostMap.h"
 #include "runtime/SslWrap.h"
 #include "pgl/RobotSym.h"
+#include "client/Client.h"
 #include "client/CltConnMgr.h"
 
 
-CltConnMgr::CltConnMgr(): thePortMgr(0), thePipeDepth(0),
+CltConnMgr::CltConnMgr(): theClient(0), thePortMgr(0), thePipeDepth(0),
 	theSslResumpProb(-1), theSslCacheLimit(-1),
 	theMinNewConnProb(-1), theConnLvlLmt(-1)  {
 }
@@ -31,7 +32,9 @@ CltConnMgr::~CltConnMgr() {
 	while (theSslSessionCache.count()) delete theSslSessionCache.pop();
 }
 
-void CltConnMgr::configure(const SockOpt &anOpt, const RobotSym *cfg, int srvCnt) {
+void CltConnMgr::configure(const SockOpt &anOpt, const Client *aClient, const int srvCnt) {
+	theClient = aClient;
+	const RobotSym *const cfg = theClient->cfg()->theRobot;
 	ConnMgr::configure(anOpt, cfg->pconnUseLmt());
 	cfg->openConnLimit(theConnLvlLmt);
 	cfg->minimizeNewConn(theMinNewConnProb);
@@ -76,7 +79,7 @@ bool CltConnMgr::atHardConnLimit() const {
 	return theConnLvlLmt >= 0 && (theConnLvl-theIdleQueue.count()) >= theConnLvlLmt;
 }
 
-Connection *CltConnMgr::get(const ObjId &oid, const NetAddr &hopAddr, ProtoIntvlPtr protoStat) {
+Connection *CltConnMgr::get(const ObjId &oid, const NetAddr &hopAddr, const NetAddr &tcpHopAddr, ProtoIntvlPtr protoStat) {
 	static const NetAddr none;
 	const NetAddr destAddr(Oid2UrlHost(oid));
 	Connection *conn = 0;
@@ -91,11 +94,12 @@ Connection *CltConnMgr::get(const ObjId &oid, const NetAddr &hopAddr, ProtoIntvl
 		theIdleQueue.dequeue(conn);
 		conn->theRd.stop(this);
 		TheFileScanner->clearTimeout(conn->sock().fd());
+		endedIdle(conn);
 	} else {
 		// check if we should close some idle conn first
 		if (theConnLvlLmt >= 0 && theConnLvl >= theConnLvlLmt)
 			closeIdle(theIdleQueue.firstOut(), ConnCloseStat::ckIdleLocal);
-		conn = open(hopAddr, protoStat, needsSsl);
+		conn = open(hopAddr, tcpHopAddr, protoStat, needsSsl);
 		if (conn && needsTunnel)
 			conn->tunnelEnd(tunnelAddr);
 	}
@@ -118,7 +122,7 @@ void CltConnMgr::closeAllIdle() {
 		closeIdle(theIdleQueue.firstOut(), ConnCloseStat::ckIdleLocal);
 }
 
-Connection *CltConnMgr::open(const NetAddr &hopAddr, ProtoIntvlPtr protoStat, bool needsSsl) {
+Connection *CltConnMgr::open(const NetAddr &hopAddr, const NetAddr &tcpHopAddr, ProtoIntvlPtr protoStat, bool needsSsl) {
 	Connection *conn = TheConnFarm.get();
 	conn->logCat(lgcCltSide);
 	conn->protoStat = protoStat;
@@ -131,7 +135,9 @@ Connection *CltConnMgr::open(const NetAddr &hopAddr, ProtoIntvlPtr protoStat, bo
 		conn->useSsl(theSslCtx, sess); // eventually
 	}
 
-	if (conn->connect(hopAddr, theSockOpt, thePortMgr)) {
+	const NetAddr &socksProxy = hopAddr != tcpHopAddr ?
+		tcpHopAddr : NetAddr();
+	if (conn->connect(hopAddr, theSockOpt, thePortMgr, socksProxy)) {
 		int limit = 1; // no pipeling by default
 		if (thePipeDepth)
 			limit = (int)MiniMax(1.1, thePipeDepth->trial(), (double)INT_MAX);
@@ -232,3 +238,11 @@ bool CltConnMgr::needMoreSslSessions() const {
 	return theSslResumpProb > 0 && (theSslCacheLimit < 0 ||
 		theSslSessionCache.count() < theSslCacheLimit);
 }
+
+bool CltConnMgr::hasCredentials() const {
+	return theClient && theClient->hasCredentials();
+}
+
+bool CltConnMgr::credentialsFor(const Connection &conn, UserCred &cred) const {
+	return theClient && theClient->credentialsFor(conn, cred);
+}
