@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -100,18 +100,24 @@ void HttpCltXact::pipeline(PipelinedCxm *aMgr) {
 	theConn->startUse();
 }
 
-void HttpCltXact::exec(Client *anOwner, Connection *aConn) {
-	CltXact::exec(anOwner, aConn);
+void HttpCltXact::exec(Connection *const aConn) {
+	CltXact::exec(aConn);
 
 	theOwner->cfg()->selectAbortCoord(theAbortCoord);
 
 	Should (!theConn->tunneling() || theConn->sslConfigured());
-	newState(theConn->tunneling() && !theConn->sslActive() ? 
-		stTunnelConnWaiting : stConnWaiting); // XXX: not last call
+	if (theConn->sslConfigured() && !theConn->sslActive()) {
+		if (theConn->tunneling())
+			theOid.connect(true);
+		else
+			theConn->sslActivate();
+	}
 
 	if (!theMgr)
 		theMgr = SingleCxm::Get(); // may be changed to PipelinedCxm later
 	theMgr->control(this);
+
+	newState(stConnWaiting);
 }
 
 void HttpCltXact::finish(Error err) {
@@ -144,7 +150,11 @@ void HttpCltXact::finish(Error err) {
 	if (err && theOwner->privCache())
 		theOwner->privCache()->purgeOid(theOid);
 
-	if (err) {
+	if (!theMgr) {
+		// no manager if lifetime timeout happens while in waiting queue
+		Must(err);
+		Must(err == errXactLifeTimeExpired);
+	} else if (err) {
 		if (err != errPipelineAbort)
 			theMgr->noteAbort(this);
 	} else {
@@ -169,8 +179,9 @@ void HttpCltXact::finish(Error err) {
 
 // XXX: the needMore value and the return value are unused
 bool HttpCltXact::controlledPostRead(bool &needMore) {
-	if (theState == stHdrWaiting || theState == stTunnelRespWaiting) {
-		if (const Error err = getHeader()) {
+	if (theState == stHdrWaiting) {
+		const Error err = getHeader();
+		if (err || (theOid.connect() && theState == stBodyWaiting)) {
 			finish(err);
 			return false;
 		}
@@ -185,7 +196,7 @@ bool HttpCltXact::controlledPostRead(bool &needMore) {
 }
 
 Error HttpCltXact::getHeader() {
-	Assert(theState == stHdrWaiting || theState == stTunnelRespWaiting);
+	Assert(theState == stHdrWaiting);
 
 	if (theRepHdr.parse(theConn->theRdBuf.content(), theConn->theRdBuf.contSize())) {
 		Error err = interpretHeader();
@@ -200,18 +211,9 @@ Error HttpCltXact::getHeader() {
 				err = handleAuth();
 			if (!err) {
 				consume(theRepHdr.theHdrSize);
-				if (theState == stTunnelRespWaiting &&
-					theRepHdr.theStatus == RepHdr::sc200_OK) {
-					theRepHdr.reset();
-					theRepSize.reset();
-					theReqSize.reset();
-					newState(stConnWaiting);
-					theMgr->rewind(this);
-				} else {
-					newState(stBodyWaiting);
-					if (theHttpStatus == RepHdr::sc407_ProxyAuthRequired)
-						saveRepHeader();
-				}
+				if (theHttpStatus == RepHdr::sc407_ProxyAuthRequired)
+					saveRepHeader();
+				newState(stBodyWaiting);
 			}
 		} else {
 			if (!err) {
@@ -345,14 +347,11 @@ bool HttpCltXact::controlledPostWrite(Size &size, bool &needMore) {
 void HttpCltXact::makeReq(WrBuf &buf) {
 	ofixedstream os(buf.space(), buf.spaceSize());
 
-	if (theState == stTunnelConnWaiting) {
-		makeConnectReq(os);
-		newState(stTunnelSpaceWaiting);
-	} else
 	if (theState == stConnWaiting) {
-		if (theConn->sslConfigured() && !theConn->sslActive())
-			theConn->sslActivate();
-		makeExplicitReq(os);
+		if (theOid.connect())
+			makeConnectReq(os);
+		else
+			makeExplicitReq(os);
 		newState(stSpaceWaiting);
 	}
 }
@@ -485,7 +484,7 @@ void HttpCltXact::makeEndToEndHdrs(ostream &os) {
 		os << hfReload;
 
 	if (theOid.range()) {
-		RangeCfg::RangesInfo res = theOwner->cfg()->makeRangeSet(os, theOid, *theRepContentCfg);
+		RangeCfg::RangesInfo res = theOwner->makeRangeSet(os, theOid, *theRepContentCfg);
 		theRangeCount = res.theCount;
 		theRangesSize = res.theTotalSize;
 	}
@@ -689,8 +688,9 @@ Error HttpCltXact::interpretHeader() {
 			return errUnsupportedControlMsg;
 	}
 
-	if (theState == stTunnelRespWaiting && theRepHdr.theStatus == RepHdr::sc200_OK) {
+	if (theOid.connect() && theRepHdr.theStatus == RepHdr::sc200_OK) {
 		theRepSize.expect(theRepHdr.theHdrSize);
+		theActualRepType = TheBodilessContentId;
 		return 0;
 	}
 
@@ -752,7 +752,9 @@ Error HttpCltXact::interpretHeader() {
 
 	theOid.repToRedir(theRepHdr.redirect());
 
-	if (theState != stTunnelRespWaiting && !theRepHdr.polyHeaders() && theRepHdr.expectPolyHeaders()) {
+	if (!theOid.connect() &&
+		!theRepHdr.polyHeaders() &&
+		theRepHdr.expectPolyHeaders()) {
 		if (const HostCfg *host = TheHostMap->at(theOid.viserv())) {
 			// do not expect Polygraph-specific headers from non-HTTP origins
 			if (host->theProtocol == Agent::pHTTP) {
@@ -1301,15 +1303,6 @@ void HttpCltXact::checkDateSync() {
 	}
 }
 
-void HttpCltXact::updatePubWorld(const ObjWorld &slice) {
-	PubWorld &pubWorld = *TheHostMap->at(theOid.viserv())->thePubWorld;
-	int sliceIdx = -1;
-	if (pubWorld.find(slice.id(), sliceIdx))
-		pubWorld.sliceAt(sliceIdx).update(slice);
-	else
-		pubWorld.add(slice);
-}
-
 // called by BodyParsers, must not finish()
 void HttpCltXact::noteContent(const ParseBuffer &content) {
 	if (theRepHdr.theChecksum.set())
