@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__CLIENT_PIPELINEDCXM_H
@@ -28,7 +28,6 @@ class PipelinedCxm: public CltXactMgr {
 		void assumeReadControl(CltXact *x, CltXactMgr *oldMgr);
 		virtual void join(CltXact *x);
 		virtual void control(CltXact *x);
-		virtual void rewind(CltXact *x);
 		virtual void release(CltXact *x); // opposite of Get()
 
 		virtual void resumeWriting(CltXact *);
@@ -43,7 +42,7 @@ class PipelinedCxm: public CltXactMgr {
 		virtual Connection *conn();
 
 	protected:
-		typedef Queue<CltXact> Line;
+		typedef Queue<CltXact, &CltXact::pipelinedXacts> Line;
 		void prepReading(CltXact *x);
 		void abortLines(CltXact *cause);
 		void abortLine(Line &line, CltXact *cause);
