@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__CLIENT_WARMUPPLAN_H
@@ -14,7 +14,6 @@ class WarmupPlan {
 	protected:
 		static bool WarmViserv(int vidx);
 		static void ReportProgress();
-		static void CheckTermination();
 		static int ReadyCount();
 
 	protected:
@@ -25,6 +24,7 @@ class WarmupPlan {
 		WarmupPlan(const Array<int> &visIdx);
 		~WarmupPlan();
 
+		int planCount() const { return thePlan.count(); };
 		int selectViserv();
 
 	protected:
