@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__CSM_CDBBODYITER_H
@@ -17,10 +17,6 @@ class CdbBodyIter: public BodyIter {
 		CdbBodyIter();
 		virtual CdbBodyIter *clone() const;
 
-		void abandonInjector();
-
-		virtual void reset();
-
 		virtual void start(WrBuf *aBuf);
 		virtual void stop();
 
