@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__CSM_GZIPENCODER_H
@@ -16,10 +16,6 @@ class GzipEncoder: public BodyIter {
 		virtual ~GzipEncoder();
 		virtual GzipEncoder *clone() const;
 
-		void abandonProducer();
-
-		virtual void reset();
-
 		virtual void start(WrBuf *aBuf);
 		virtual void stop();
 
@@ -29,7 +25,7 @@ class GzipEncoder: public BodyIter {
 		virtual bool pourBody();
 
 	protected:
-		int theLevel;          // compression level: 0-9
+		const int theLevel;    // compression level: 0-9
 		BodyIter *theProducer; // produces unencoded content
 		Deflator *theEncoder;  // compresses content
 
