@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__CSM_OBJLIFECYCLE_H
@@ -23,15 +23,6 @@ class ObjLifeCycle {
 
 		void configure(const ObjLifeCycleSym *cfg);
 
-		//const RndDistr *modTime() const { return theModTime; }
-		//const RndDistr *nowExpTime() const { return theNowExpTime; }
-		//const RndDistr *modExpTime() const { return theModExpTime; }
-
-		void modVar(double aVar);
-		void modTime(RndDistr *aDistr);
-		void nowExpTime(RndDistr *aDistr);
-		void modExpTime(RndDistr *aDistr);
-
 		void calcTimes(int seed, ObjTimes &times);
 
 	protected:
