@@ -1,12 +1,12 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__CSM_XMLTAGIDENTIFIER_H
 #define POLYGRAPH__CSM_XMLTAGIDENTIFIER_H
 
-#include "xstd/StrIdentifier.h"
+#include "xstd/PrefixIdentifier.h"
 
 // identifies tags (and their attributes) of interest
 class XmlTagIdentifier {
@@ -28,8 +28,8 @@ class XmlTagIdentifier {
 		XmlTagIdentifier(const XmlTagIdentifier&);
 
 	protected:
-		StrIdentifier theTagIdx;
-		Array<StrIdentifier*> theAttrIndeces;
+		PrefixIdentifier theTagIdx;
+		Array<PrefixIdentifier*> theAttrIndeces;
 };
 
 #endif
