@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__CSM_CDBBUILDERS_H
@@ -9,6 +9,7 @@
 class ContentDbase;
 class CdbEntry;
 class CdbePage;
+class PrefixIdentifier;
 
 class CdbBuilder {
 	public:
@@ -42,7 +43,7 @@ class MarkupParser: public CdbBuilder {
 		const String *AttrValReplacement(const String &keyPfx, const String &keySfx);
 
 	private:
-		static StrIdentifier *TheReplIdentifier;
+		static PrefixIdentifier *TheReplIdentifier;
 		static Array<String*> TheReplacements;
 };
 
