@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__DNS_DNSMSG_H
@@ -12,7 +12,7 @@
 class DnsMsg {
 	public:
 		enum RCodes { rcodeMask = 0x07, rcodeFmt = 1, rcodeSrvFail, rcodeNoName, rcodeNoSupp, rcodeRefused };
-		enum Flags { flagQuery  = (0 << 15), flagResp = (1 << 15), flagAQuery = (0 << 11) };
+		enum Flags { flagQuery  = (0 << 15), flagResp = (1 << 15), flagAQuery = (0 << 11), flagRecursionDesired = (1 << 8) };
 		enum Types { typeA = 1, typeAAAA = 28 };
 		enum Classes { classIn = 1 };
 
