@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -35,7 +35,7 @@ bool DnsQuery::dumpAQuery(OBStream &os) const {
 	os << id(); // XXX: check that we dump as short
 
 	// XXX: check that we dump as short
-	const Field flags = flagQuery | flagAQuery;
+	const Field flags = flagQuery | flagAQuery | flagRecursionDesired;
 	os << flags;
 
 	os << (Field)1; // QDCOUNT
