@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__LOGANALYZERS_LEVELTRACEFIG_H
@@ -22,7 +22,8 @@ class LevelTraceFig: public ReportTraceFigure {
 	public:
 		LevelTraceFig();
 
-		void stats(const Stex *aStex1, const Stex *aStex2, const PhaseInfo *phase);
+		void stats(const LevelStex *const aStex, const PhaseInfo *const phase);
+		void compareWith(const LevelStex *const stex);
 
 	protected:
 		virtual int createCtrlFile();
@@ -32,8 +33,8 @@ class LevelTraceFig: public ReportTraceFigure {
 		int dumpDataLine(const LevelStex *stex, Time stamp, const StatIntvlRec &r);
 		
 	protected:
-		const Stex *theStex1;
-		const Stex *theStex2;
+		const LevelStex *theStex;
+		Array<const LevelStex *> theComparison; // theStex and others
 		const PhaseInfo *thePhase;
 		const PhaseTrace *theTrace;
 };
