@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -470,6 +470,7 @@ void SideInfo::compileStats(BlobDb &db, const PhaseInfo &phase, const Scope &sco
 	addMeasBlob(db, "rep.bwidth" + sfx, stats.repBwidth()/(1024*1024/8), "Mbits/sec", "response bandwidth");
 	addMeasBlob(db, "rep.rptm.mean" + sfx, Time::Secd(stats.repTime().mean()/1000.), "mean response time");
 
+	cmpProtoStats(db, phase, &StatIntvlRec::theSocksStat, scope);
 	cmpProtoStats(db, phase, &StatIntvlRec::theSslStat, scope);
 	cmpProtoStats(db, phase, &StatIntvlRec::theFtpStat, scope);
 
@@ -498,6 +499,7 @@ void SideInfo::compileStats(BlobDb &db, const PhaseInfo &phase, const Scope &sco
 	addMeasBlob(db, "url.recurrence.ratio" + sfx, stats.recurrenceRatio(), "%", "recurrence ratio");
 
 	cmplLoadBlob(db, scope);
+	cmplProtoLoadBlob(db, phase, &StatIntvlRec::theSocksStat, scope);
 	cmplProtoLoadBlob(db, phase, &StatIntvlRec::theSslStat, scope);
 	cmplProtoLoadBlob(db, phase, &StatIntvlRec::theFtpStat, scope);
 	cmplRptmFigure(db, scope);
@@ -954,6 +956,7 @@ void SideInfo::cmplConnLevelTable(BlobDb &db, const PhaseInfo &phase, const Scop
 
 	cmplLevelTableRec(db, "conn.open.", "open", stats.theOpenLvl, scope, table);
 	cmplLevelTableRec(db, "conn.estb.", "established", stats.theEstbLvl, scope, table);
+	cmplLevelTableRec(db, "conn.idle.", "idle", stats.theIdleLvl, scope, table);
 
 	blob << table;
 
@@ -976,9 +979,12 @@ void SideInfo::cmplConnLevelTable(BlobDb &db, const PhaseInfo &phase, const Scop
 void SideInfo::cmplConnLevelFigure(BlobDb &db, ReportBlob &blob, const Scope &scope) {
 	LevelStex stex1("open", "open", &StatIntvlRec::theOpenLvl);
 	LevelStex stex2("estb", "established", &StatIntvlRec::theEstbLvl);
+	LevelStex stex3("idle", "idle", &StatIntvlRec::theIdleLvl);
 	LevelTraceFig fig;
 	fig.configure("conn.level.trace" + scope, "concurrent HTTP/TCP connection level trace");
-	fig.stats(&stex1, &stex2, &phase(scope));
+	fig.stats(&stex1, &phase(scope));
+	fig.compareWith(&stex2);
+	fig.compareWith(&stex3);
 	fig.globalStart(theTest->startTime());
 	const String &figKey = fig.plot(db).key();
 	blob << db.include(figKey);
@@ -1133,7 +1139,7 @@ void SideInfo::cmplPopulLevelFigure(BlobDb &db, ReportBlob &blob, const Scope &s
 	LevelStex stex1("populus", "agents", &StatIntvlRec::thePopulusLvl);
 	LevelTraceFig fig;
 	fig.configure("populus.level.trace" + scope, "population level trace");
-	fig.stats(&stex1, 0, &phase(scope));
+	fig.stats(&stex1, &phase(scope));
 	fig.globalStart(theTest->startTime());
 	const String &figKey = fig.plot(db).key();
 	blob << db.include(figKey);
@@ -1189,7 +1195,8 @@ void SideInfo::cmplXactLevelFigure(BlobDb &db, ReportBlob &blob, const Scope &sc
 	LevelStex stex2("wait", "waiting", &StatIntvlRec::theWaitLvl);
 	LevelTraceFig fig;
 	fig.configure("xact.level.trace" + scope, "concurrent HTTP transaction level trace");
-	fig.stats(&stex1, &stex2, &phase(scope));
+	fig.stats(&stex1, &phase(scope));
+	fig.compareWith(&stex2);
 	fig.globalStart(theTest->startTime());
 	const String &figKey = fig.plot(db).key();
 	blob << db.include(figKey);
@@ -1669,7 +1676,8 @@ void SideInfo::cmplStreamTableRec(BlobDb &db, StatTable &table, const Stex &stex
 
 	XmlTableRec tr;
 
-	XmlTableHeading th(stex.name());
+	XmlTableHeading th;
+	th << db.ptr("object." + stex.key() + scope, XmlText(stex.name()));
 	th << algnLeft;
 	tr << th;
 
@@ -2773,6 +2781,8 @@ void SideInfo::Configure() {
 		&StatPhaseRec::thePostXacts, &StatIntvlRec::thePost), allMethods);
 	AddStex(TheReplyStex, new SimpleStex("method_put", "PUT",
 		&StatPhaseRec::thePutXacts, &StatIntvlRec::thePut), allMethods);
+	AddStex(TheReplyStex, new SimpleStex("method_connect", "CONNECT",
+		&StatPhaseRec::theConnectXacts, &StatIntvlRec::theConnect), allMethods);
 	AddStex(TheReplyStex, allMethods, TheAllReps);
 
 	AddStex(TheReplyStex, TheAllReps, 0);
@@ -2799,6 +2809,7 @@ void SideInfo::Configure() {
 	AddStex(TheReplyStex, new ValidationMissStex("useless_proxy_validations", "useless proxy validations", &StatPhaseRec::theProxyValidationR), proxyValidation);
 	AddStex(TheReplyStex, proxyValidation, TheAllReps);
 
+	AddProtoStexes(&StatIntvlRec::theSocksStat);
 	AddProtoStexes(&StatIntvlRec::theSslStat);
 	AddProtoStexes(&StatIntvlRec::theFtpStat);
 
@@ -2878,6 +2889,9 @@ void SideInfo::Configure() {
 	AddStex(TheCompoundRequestStex, new CompoundRequestStex("compound.auth.not.req", "isolated requests",
 		&StatPhaseRec::theIsolated), TheAllReqs);
 
+	AddStex(TheReplyStex, new SimpleStex("custom_stats", "custom",
+		&StatPhaseRec::theCustomXacts, &StatIntvlRec::theCustom), 0);
+
 	Stex *allReplyStatus = new AllStatusCodeStex("reply_status.all", "All HTTP reply status codes", &StatPhaseRec::theStatusCode);
 	for (int i = 0; i <= StatusCodeStat::scsMaxValue; ++i) {
 		const String label(AnyToString(i));
