@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -332,6 +332,7 @@ const TmSzHistStat *AllMethodsStex::hist(const PhaseInfo &phase) const {
 		theXactHist += phase.stats().theHeadXacts;
 		theXactHist += phase.stats().thePostXacts;
 		theXactHist += phase.stats().thePutXacts;
+		theXactHist += phase.stats().theConnectXacts;
 		return &theXactHist;
 	} else {
 		return 0;
@@ -339,7 +340,7 @@ const TmSzHistStat *AllMethodsStex::hist(const PhaseInfo &phase) const {
 }
 
 const TmSzStat *AllMethodsStex::trace(const StatIntvlRec &rec) const {
-	theXactAggr = rec.theHead + rec.thePost + rec.thePut;
+	theXactAggr = rec.theHead + rec.thePost + rec.thePut + rec.theConnect;
 	return &theXactAggr;
 }
 
