@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__PGL_AGENTSYM_H
@@ -10,7 +10,6 @@
 #include "pgl/PglNumSym.h"
 #include "pgl/HostsBasedSym.h"
 
-class StrIdentifier;
 class StringSym;
 class SocketSym;
 class PopModelSym;
@@ -29,16 +28,18 @@ class AgentSym: public HostsBasedSym {
 
 		String kind() const;
 		UniqId world() const;
-		RndDistr *httpVersions(const StrIdentifier &versionNames) const;
+		RndDistr *httpVersions(const TokenIdentifier &versionNames) const;
 		SocketSym *socket() const;
+		RndDistr *xactLifetime() const;
 		RndDistr *xactThink() const;
 		RndDistr *pconnUseLmt() const;
 		Time idlePconnTimeout() const;
 		PopModelSym *popModel() const;
 		bool msgTypes(Array<StringSym*> &types, Array<double> &tprobs) const;
-		RndDistr *msgTypes(const StrIdentifier &typeNames) const;
+		RndDistr *msgTypes(const TokenIdentifier &typeNames) const;
 		bool abortProb(double &p) const;
 		bool cookieSender(double &prob) const;
+		bool customStatsScope(Array<StringSym*> &syms) const;
 		bool sslWraps(Array<SslWrapSym*> &syms, RndDistr *&sel) const;
 
 	protected:
