@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__PGL_CONTENTSYM_H
@@ -9,6 +9,7 @@
 #include "xstd/Array.h"
 #include "pgl/PglRecSym.h"
 
+class ClientBehaviorSym;
 class MimeSym;
 class RndDistr;
 class ObjLifeCycleSym;
@@ -36,6 +37,7 @@ class ContentSym: public RecSym {
 		bool contains(Array<ContentSym*> &ccfgs, RndDistr *&selector, Array<double> &cprobs) const;
 		RndDistr *embedCount() const;
 		bool choiceSpace(int &space) const;
+		ClientBehaviorSym *clientBehavior() const;
 		String cdb() const;
 
 		String injectDb() const;
