@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "pgl/pgl.h"
@@ -33,8 +33,13 @@ SynSym *DynamicNameSym::dupe(const String &type) const {
 }
 
 void DynamicNameSym::val(const NetAddr &addr) {
-	Assert(addr.addrA().startsWith("*."));
-	NetAddrSym::val(NetAddr(addr.addrA()(1, String::npos), addr.port()));
+	if (addr.addrA().startsWith("*."))
+		NetAddrSym::val(NetAddr(addr.addrA()(1, String::npos), addr.port()));
+	else
+	if (addr.addrA().startsWith("."))
+		NetAddrSym::val(addr);
+	else
+		NetAddrSym::val(NetAddr("." + addr.addrA(), addr.port()));
 }
 
 void DynamicNameSym::setNewProb(const double  prob) {
