@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "pgl/pgl.h"
@@ -59,6 +59,22 @@ PglRec *PglRec::clone() const {
 	return rec;
 }
 
+// Note: parameter types are not checked
+void PglRec::copyCommon(const PglRec &rec) {
+	for (int i = 0; i < count(); ++i) {
+		SynSymTblItem *val = 0;
+		if (!rec.find(keyAt(i), val) || !val)
+			continue;
+
+		SynSymTblItem *const clone = new SynSymTblItem(*val);
+		if (const SynSym *const s = val->sym())
+			clone->sym(s->clone(s->type()));
+
+		delete valAt(i);
+		valAt(i) = clone;
+	}
+}
+
 SynSymTblItem *PglRec::bAdd(const String &type, const String &name, SynSym *s) {
 	SynSymTblItem *member = new SynSymTblItem(type, name);
 	member->sym(s);
