@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__PGL_PGLSEMX_H
@@ -30,6 +30,7 @@ class NetAddrSym;
 class NetAddrRangeSym;
 class QualifSym;
 class DistrSym;
+class DynamicNameSym;
 class PopDistrSym;
 class RegExSym;
 
@@ -129,6 +130,8 @@ class PglSemx {
 		ContainerSym *selectItems(const String &cname, const ContainerSym &items, int goalCount, const TokenLoc &loc);
 		ContainerSym *setAddrPort(const String &cname, const ContainerSym &items, int newPort, const TokenLoc &loc);
 		RegExSym *orItems(const ContainerSym &items);
+		DynamicNameSym *dynamicName(const NetAddrSym &addr, const NumSym &prob);
+		ContainerSym *dynamize(const String &cname, const ContainerSym &items, const NumSym &prob);
 
 		String typeName(const ParsSym &pgl) const;
 		String objName(const ParsSym &pgl) const;
