@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__PGL_SERVERSYM_H
@@ -31,7 +31,7 @@ class ServerSym: public AgentSym {
 		RndDistr *cookieSetCount() const;
 		RndDistr *cookieValueSize() const;
 		bool reqBodyAllowed(double &f) const;
-		RndDistr *protocols(const StrIdentifier &protocolNames) const;
+		RndDistr *protocols(const TokenIdentifier &protocolNames) const;
 
 	protected:
 		virtual SynSym *dupe(const String &dType) const;
