@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__PGL_SSLWRAPSYM_H
@@ -10,7 +10,6 @@
 #include "xstd/Size.h"
 #include "pgl/PglRecSym.h"
 
-class StrIdentifier;
 
 // a set of addresses that talk SSL and parameters for SSL protocol
 class SslWrapSym: public RecSym {
@@ -25,12 +24,14 @@ class SslWrapSym: public RecSym {
 
 		String rootCertificate() const;
 		String sharingGroup() const;
-		RndDistr *protocols(const StrIdentifier &protocolNames) const;
+		bool protocols(Array<String*> &protos, Array<double> &probs) const;
+		RndDistr *protocols(const TokenIdentifier &protocolNames) const;
 		bool rsaKeySizes(Array<Size>&, RndDistr *&) const;
 		bool ciphers(Array<String*> &, RndDistr *&) const;
 		bool sessionResumpt(double &prob) const;
 		bool sessionCacheSize(int &count) const;
 		String sslConfigFile() const;
+		bool verifyPeerCertificate(bool &set) const;
 
 	protected:
 		virtual SynSym *dupe(const String &dType) const;
