@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -62,7 +62,7 @@ StatPhase::StatPhase(): theRecs(lgcEnd), theMgr(0),
 		theRecs.push(new StatPhaseRec);
 
 	for (int i = 0; i < theLocks.capacity(); ++i)
-		theLocks.push(false);
+		theLocks.push(0);
 
 	theChannels << TheInfoChannel;
 }
@@ -199,19 +199,16 @@ void StatPhase::finalizeStats() {
 }
 
 void StatPhase::lock(const LockType lt) {
-	if (!locked(lt))
-		theLocks[lt] = true;
-	else {
-		Comment(0)
-			<< "internal error: attempt to lock a locked lock '"
-			<< TheLockNames[lt]
-			<< "' (salvaged)" << endc;
-	}
+	++theLocks[lt];
+	Comment(10) << "fyi: current lock level for '"<< TheLockNames[lt]
+		<< "' increased to " << theLocks[lt] << endc;
 }
 
 void StatPhase::unlock(const LockType lt) {
 	if (locked(lt)) {
-		theLocks[lt] = false;
+		--theLocks[lt];
+		Comment(10) << "fyi: current lock level for '"<< TheLockNames[lt]
+			<< "' decreased to " << theLocks[lt] << endc;
 		if (!locked())
 			checkpoint();
 	} else {
@@ -337,6 +334,9 @@ void StatPhase::noteXactEvent(BcastChannel *ch, const Xaction *x) {
 		const bool authing = x->authing();
 
 		// stats must be recorded in only one category for totals to work!
+		if (oid.aborted())
+			; // do nothing, StatIntvl has enough(?) stats for these
+		else
 		if (authing) {
 			Assert(x->logCat() == lgcCltSide);
 			// do not record anything for authing transactions
@@ -348,6 +348,9 @@ void StatPhase::noteXactEvent(BcastChannel *ch, const Xaction *x) {
 		if (oid.active() || oid.passive())
 			rec.theFtpXacts.record(repTime, repSize, oid.active());
 		else
+		if (oid.connect())
+			rec.theConnectXacts.record(repTime, repSize);
+		else
 		if (oid.repToRedir())
 			rec.theRepToRedirXacts.record(repTime, repSize);
 		else
@@ -375,9 +378,6 @@ void StatPhase::noteXactEvent(BcastChannel *ch, const Xaction *x) {
 		else
 		if (oid.put())
 			rec.thePutXacts.record(repTime, repSize);
-		else
-		if (oid.aborted())
-			; // do nothing, StatIntvl has enough(?) stats for these
 		else {
 			Should(false); // all categories should be accounted for
 		}
@@ -429,8 +429,11 @@ void StatPhase::noteXactEvent(BcastChannel *ch, const Xaction *x) {
 
 	if (ch == TheXactEndChannel ||
 		ch == TheXactRetrChannel ||
-		ch == TheXactErrChannel)
+		ch == TheXactErrChannel) {
 		rec.theStatusCode.record(x->httpStatus(), repTime, repSize);
+		if (x->inCustomStatsScope())
+			rec.theCustomXacts.record(repTime, repSize);
+	}
 
 	StatIntvl::noteXactEvent(ch, x);
 }
