@@ -1,13 +1,13 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
 
 #include "xstd/String.h"
 #include "xstd/Clock.h"
-#include "xstd/StrIdentifier.h"
+#include "xstd/PrefixIdentifier.h"
 #include "xstd/gadgets.h"
 #include "base/OLog.h"
 #include "base/AddrParsers.h"
@@ -30,7 +30,7 @@ class MsgHdrParsTab {
 		~MsgHdrParsTab();
 
 	public:
-		StrIdentifier *ids;
+		PrefixIdentifier *ids;
 		Array<MsgHdr::Parser*> *parsers;
 };
 
@@ -238,7 +238,7 @@ bool MsgHdr::ParseAuthenticate(const char *&buf, const char *eoh, AuthChallenge
 	// Header fields are scanned in the reverse order of appearance so the
 	// last header field we see and honor is the first one in the header.
 
-	static StrIdentifier authSchemeIdentifier;
+	static PrefixIdentifier authSchemeIdentifier;
 	if (!authSchemeIdentifier.count()) {
 		authSchemeIdentifier.add("Basic", authBasic);
 		authSchemeIdentifier.add("NTLM", authNtlm);
@@ -837,7 +837,7 @@ void RepHdr::Clean() {
 /* MsgHdrParsTab */
 
 MsgHdrParsTab::MsgHdrParsTab() {
-	ids = new StrIdentifier;
+	ids = new PrefixIdentifier;
 	parsers = new Array<MsgHdr::Parser*>;
 }
 
