@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__RUNTIME_POLYBCASTCHANNELS_H
@@ -22,6 +22,8 @@ extern BcastChannel *TheSessionEndChannel;
 extern BcastChannel *TheConnOpenChannel;
 extern BcastChannel *TheConnCloseChannel;
 extern BcastChannel *TheConnEstChannel;
+extern BcastChannel *TheConnIdleBegChannel;
+extern BcastChannel *TheConnIdleEndChannel;
 extern BcastChannel *TheConnSslActiveChannel;
 extern BcastChannel *TheConnSslInactiveChannel;
 
