@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "base/polygraph.h"
@@ -61,6 +61,8 @@ void HttpSrvXact::noteBodyDataReady() {
 		newState(stSpaceWaiting);
 		if (theLastReqByteTime < 0)
 			theLastReqByteTime = TheClock - theStartTime;
+		if (the100ContinueState == csAllowed)
+			the100ContinueState = csDone; // send no "100 Continue" if got all
 	} else
 	if (theConn->atEof()) { // premature eof
 		if (theHttpStatus == RepHdr::sc417_ExpectationFailed) {
@@ -75,7 +77,9 @@ void HttpSrvXact::noteBodyDataReady() {
 			finish(errPrematureEof);
 			return;
 		}
-	}
+	} else // client probably wants to send more body data
+	if (the100ContinueState == csAllowed || the100ContinueState == csDenied)
+		newState(stSpaceWaiting); // but we need to respond with 100 or 417
 }
 
 void HttpSrvXact::noteBufReady() {
@@ -92,7 +96,7 @@ void HttpSrvXact::noteHdrDataReady() {
 			return;
 		}
 		Assert(the100ContinueState != csDone);
-		newState(the100ContinueState == csNone ? stBodyWaiting : stSpaceWaiting);
+		newState(stBodyWaiting); // may become stSpaceWaiting to send 100/417
 	} else
 	if (theConn->theRdBuf.full()) { // header too big
 		finish(errHugeHdr);
@@ -378,16 +382,6 @@ Error HttpSrvXact::setTarget(const NetAddr &) {
 	return 0;
 }
 
-void HttpSrvXact::updatePubWorld(const ObjWorld &newSlice) {
-	PubWorld &world = *TheHostMap->findPubWorldAt(theOid.viserv());
-
-	int sliceIdx;
-	if (world.find(newSlice.id(), sliceIdx))
-		world.sliceAt(sliceIdx).update(newSlice);
-	else
-		world.add(newSlice);
-}
-
 // take all ranges from theReqHdr and do "normalization"
 // That is:
 // 1. Remove unsatisfiable ranges.
@@ -654,7 +648,9 @@ void HttpSrvXact::put2xxContentHead(ostream &os) {
 	if (theRepContentCfg->multipleContentCodings())
 		os << hfVaryAcceptEncoding;
 
-	// XXX: no checksum for 206 until ContentCfg::calcChecksum is fixed
+	// Servers probably MUST NOT send a Content-MD5 header with
+	// 206 responses. See
+	// http://trac.tools.ietf.org/wg/httpbis/trac/ticket/178
 	if (!theOid.range() && theRepContentCfg->calcChecksumNeed(theOid))
 		putChecksum(*theRepContentCfg, theOid, os);
 
