@@ -1,17 +1,16 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #ifndef POLYGRAPH__SERVER_SRVXACT_H
 #define POLYGRAPH__SERVER_SRVXACT_H
 
-#include "xstd/AlarmClock.h"
 #include "runtime/Xaction.h"
 
 class Server;
 
-class SrvXact: public Xaction, public AlarmUser {
+class SrvXact: public Xaction {
 	public:
 		SrvXact();
 
@@ -44,6 +43,7 @@ class SrvXact: public Xaction, public AlarmUser {
 
 	protected:
 		Server *theOwner;
+		Alarm theThinkTimeAlarm; // think time timeout alarm
 };
 
 #endif
