@@ -1,6 +1,6 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "xparser/xparser.h"
@@ -24,6 +24,11 @@ SynSymTblItem::~SynSymTblItem() {
 	delete theSym;
 }
 
+void SynSymTblItem::sym(SynSym *aSym) {
+	theSym = aSym;
+	if (theSym && !theSym->loc())
+		theSym->loc(loc());
+}
 
 
 /* SynSymTbl */
