@@ -1,12 +1,11 @@
 
 /* Web Polygraph       http://www.web-polygraph.org/
- * (C) 2003-2006 The Measurement Factory
+ * Copyright 2003-2011 The Measurement Factory
  * Licensed under the Apache License, Version 2.0 */
 
 #include "xstd/xstd.h"
 
 #include "xstd/h/iostream.h"
-#include "xstd/h/sstream.h"
 #include "xstd/h/string.h"
 
 #include "xstd/Assert.h"
@@ -261,12 +260,6 @@ String String::operator +(char c) const {
 
 /* operators on strings */
 
-ostream &operator <<(ostream &os, const String &s) {
-	if (s)
-		os.write(s.data(), s.len());
-	return os;
-}
-
 String operator +(const String &s1, const String &s2) {
 	String s(s1);
 	s += s2;
